// This is -*- C++ -*-
// $Id: draw.cc,v 1.13 1998/07/21 20:58:19 trow Exp $

/* 
 * draw.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

 

#include <string.h>
#include <gdk/gdkprivate.h>
#include "draw.h"

Gdk_Draw::Gdk_Draw() : d1_(0), d2_(0), 
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{ }

Gdk_Draw::Gdk_Draw(GdkDrawable* d) : d1_(d), d2_(0),
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{
  // It's really OK for the args to be NULL, e.g. when copying
  //  the drawables from another Gdk_Draw
}

Gdk_Draw::Gdk_Draw(GdkDrawable* d1, GdkDrawable* d2) : d1_(d1), d2_(d2),
  xo_(0), yo_(0), w_(-1), h_(-1),
  gc_(0)
{
  // ditto.
}

Gdk_Draw::~Gdk_Draw()
{
  // clean up our GC stack
  delete gc_;
  for(list<Gdk_GC*>::iterator i=gc_stack_.begin(); i != gc_stack_.end(); ++i)
    delete *i;
}

void
Gdk_Draw::set_drawable(GdkDrawable* d)
{
  if (d == 0)
    g_warning("Passed null pointer to Gdk_Draw::set_drawable");
  d1_ = d;

  // When we set change our drawable, clear out any old GC cruft...
  gc_flush_stack();
}

void
Gdk_Draw::set_secondary_drawable(GdkDrawable* d)
{
  if (d == 0)
    g_warning("Passed null pointer to Gdk_Draw::set_drawable");
  d2_ = d;
}

gint
Gdk_Draw::width()
{
  g_return_val_if_fail((w_ > 0) || (d1_ != 0), 0);
  return w_ > 0 ? w_ : ((GdkWindowPrivate*)drawable())->width;
}

gint
Gdk_Draw::height()
{
  g_return_val_if_fail((h_ > 0) || (d1_ != 0), 0);
  return h_ > 0 ? h_ : ((GdkWindowPrivate*)drawable())->height;
}

void
Gdk_Draw::gc_save()
{
  if (gc_) {
    gc_stack_.push_back(gc());
    gc_ = 0;
  }
}

void
Gdk_Draw::gc_clone()
{
  Gdk_GC* clone = new Gdk_GC(*gc());
  gc_save();
  gc_ = clone;
}

void
Gdk_Draw::gc_restore()
{
  if (gc_)
    delete gc_;
  gc_ = 0;
  if (!gc_stack_.empty()) {
    gc_ = gc_stack_.back();
    gc_stack_.pop_back();
  }
}

void
Gdk_Draw::gc_flush_stack()
{
  do { gc_restore(); } while (!gc_stack_.empty());
}

void 
Gdk_Draw::gc_push(Gdk_GC* pushme)
{
  gc_save();
  gc_ = pushme;
}

Gdk_GC*
Gdk_Draw::gc_pop()
{
  Gdk_GC* popme = gc();
  gc_restore();
  return popme;
}


GdkPixmap*
Gdk_Draw::copy_rectangle_to_pixmap(gint x, gint y, gint w, gint h, 
				       Gdk_GC* GC)
{
  if (GC == 0) GC = gc();
  GdkPixmap* pix = gdk_pixmap_new(drawable(), w, h, -1);
  gdk_draw_pixmap(pix, GC->gc(), drawable(), x, y, 0, 0, w, h);
  return pix;
}

void
Gdk_Draw::paste_pixmap(GdkPixmap* pix, gint x, gint y, Gdk_GC* GC)
{
  if (GC == 0) GC = gc();
  gdk_draw_pixmap(drawable(), GC->gc(), pix, 0, 0, x, y,
		  ((GdkWindowPrivate*)pix)->width,
		  ((GdkWindowPrivate*)pix)->height);

  if (secondary_drawable())
    gdk_draw_pixmap(secondary_drawable(), GC->gc(), pix, 0, 0, x, y,
		    ((GdkWindowPrivate*)pix)->width,
		    ((GdkWindowPrivate*)pix)->height);

}

void
Gdk_Draw::draw_aligned_string(gint x, gint y, const gchar* str,
			      gint x_align, gint y_align,
			      gint x_adjust, gint y_adjust,
			      Gdk_GC* GC)
{
  if (GC == 0) GC = gc();

  gint tw = GC->string_width(str);
  if (x_align < 0)
    x -= tw + x_adjust;
  else if (x_align == 0)
    x -= tw/2;
  else
    x += x_adjust;

  gint th = GC->font_height();
  if (y_align == 0)
    y += th/2;
  else if (y_align < 0)
    y -= y_adjust;
  else
    y += th + y_adjust;

  draw_string(x,y,str,GC);
}

void
Gdk_Draw::draw_text(gint x, gint y, const gchar* str, gint text_align,
		    Gdk_GC* GC)
{
  if (GC == 0) GC = gc();
  gint h = GC->font_height();
  gchar* ss = strdup(str);
  gchar* ss_orig = ss;

  gint w=0;
  if (text_align >= 0) 
    w = GC->text_width(str);

  while (*ss) {
    gchar* p = ss;
    while (*p && *p != '\n') ++p;
    bool more_to_do = *p;
    *p = '\0';

    gint fudge = 0;
    if (text_align >= 0) {
      gint lw = GC->string_width(ss);
      if (text_align > 0)
	fudge = w - lw;
      else
	fudge = (w - lw)/2;
    }

    draw_string(x + fudge, y, ss, GC);
    y += h;
    ss = p;
    if (more_to_do) ++ss;
  }

  free(ss_orig);
}

void
Gdk_Draw::draw_aligned_text(gint x, gint y, const gchar* str,
			    gint text_align,
			    gint x_align, gint y_align,
			    gint x_adjust, gint y_adjust,
			    Gdk_GC* GC)
{
  if (GC == 0) GC = gc();
  
  gint tw = GC->text_width(str);
  if (x_align < 0)
    x -= tw + x_adjust;
  else if (x_align == 0)
    x -= tw/2;
  else
    x += x_adjust;

  y += GC->font_height();
  gint th = GC->text_height(str);
  if (y_align == 0)
    y -= th/2;
  else if (y_align < 0)
    y -= th + y_adjust;
  else
    y += y_adjust;

  draw_text(x,y,str,text_align,GC);
}




// $Id: draw.cc,v 1.13 1998/07/21 20:58:19 trow Exp $
